'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard   www.joseng.de                                       |
'| Datei: Simodrive.bas                                                        |
'| Beschreibung: Firmware zum Steuern eine Simodrive FM Motorsteuerung von     |
'|               Siemens mit Stepsrive Motor                                   |
'|                                                                             |
'| Fuse Bits:  Low Byte: 0xFF           HighByte: 0xD9                         |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Befehle :                                                                   |
'|                                                                             |
'| - s: Steps/Schritte die gefahren werden sollen                              |
'| - c: Current/Strom des Motors (0-100%)                                      |
'| - f: Frequency/Frequenz mit der die Schritte gefahren werden                |
'| - r: Statusrelaisfunktion festlegen                                         |
'| - v: Version der Software                                                   |
'| - d: Direction/Richtung in die der Motor dreht                              |
'| - e: Fahrmodus was gefahren werden soll                                     |
'|                                                                             |
'+-----------------------------------------------------------------------------+

'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Informationen :                                                             |
'|                                                                             |
'| Die SIMODRIVE Steuerung wurde auf 1000 Schritte pro Umdrehung eingestellt   |
'| und der Motor kann bis 1200 Umdrehungen pro Minute                          |
'| Das wren 1200 * 1000 Schritte pro Minute und 20000 Schritte/Sekunde        |
'|                                                                             |
'+-----------------------------------------------------------------------------+


$regfile = "m8def.dat"                                                'ATmega8
$crystal = 16000000                                                   '16MHz Quarz
$baud = 19200                                                         'Baudrate 19,2Kbaud


'Funktionen
Declare Sub Interpreter()
Declare Sub Wait_ready()


'Interrupt Not-Aus (INT0)
Config Int0 = Falling
On Int0 Isr_not_aus
Enable Int0


'Interrupt ext. Clock (INT1)
Config Int1 = Rising
On Int1 Isr_int1
'Enable Int1


'Serielle Schnittstelle RS485
Config Print0 = Portd.4 , Mode = Set


'Timer0 (Fahrtimer)   100kH
Config Timer0 = Timer , Prescale = 1
On Timer0 Isr_timer0
'Enable Timer0


'Timer1 (Sekundentimer)
Config Timer1 = Timer , Prescale = 256
On Timer1 Isr_timer1
Enable Timer1


'Timer2 (PWM 8bit) (16MHz / 1024 ~ 16kHz)
Config Timer2 = Pwm , Pwm = On , Prescale = 1024 , Compare Pwm = Clear Down
'Enable Timer2


'Ein/Ausgnge festlegen
Config Portd.4 = Output                                               'RS485 Busrichtung
Config Portc.0 = Input                                                'ext. Dir
Config Portc.1 = Output                                               'Statusrelais
Config Portd.5 = Input                                                'Ready von Motorsteuerung (invertiert)
Config Portd.6 = Input                                                'Motor steht von Motorsteuerung (invertiert)
Config Portd.7 = Output                                               'Gate (Freigabe) fr Motorsteuerung
Config Portb.0 = Output                                               'Puls fr Motorsteuerung
Config Portb.1 = Output                                               'Richtung fr Motorsteuerung
Config Portb.2 = Output                                               'Enable fr Motorsteuerung
Config Portb.3 = Output                                               'PWM (Motorstrom) fr Motorsteuerung
Config Portb.4 = Output                                               'RunLED
Config Portd.2 = Input                                                'Notaus
Config Portd.3 = Input                                                'ext. Pulse


'Aliasdeklarationen
Motorpuls Alias Portb.0
Motordir Alias Portb.1
Motorenable Alias Portb.2
Runled Alias Portb.4
Motorgate Alias Portd.7
Motorstill Alias Pind.6                                               'Invertiert!
Motorready Alias Pind.5                                               'Invertiert!
Statusrelais Alias Portc.1
Extdir Alias Pinc.0
Notaus Alias Pind.2                                                   'Invertiert, low = Notaus gedrckt


'Deklarationen
Const Timervorgabe0 = 96                                              'Timer0 20kh
Const Timervorgabe1 = 3036                                            'Timer1 1Hz
Dim I As Integer                                                      'Zhldummy
Dim Modulname As String * 10                                          'Name des Moduls
Dim Buffer As String * 50                                             'Eingangspuffer der Seriellen
Dim Schritte As Integer                                               'Vorgegebene Schrittzahl
Dim Re_schritte As Integer                                            'Schritte die noch zu fahren sind
Dim Drehzahl As Integer                                               'Die Drehzahl in Umdrehungen/min mit der die Schritte gefahren werden
Dim Richtung As Bit                                                   'Die Motorrichtung die per Komando gesetzt wurde
Dim Relaisfunktion As Byte                                            'Legt die Funktion des Statusrelais fest
Dim Laenge As Byte                                                    'Lnge eines Strings (wird universal eingesetzt)
Dim Modul As String * 10                                              'Name des angesprochenen Moduls
Dim Kommando As String * 5                                            'Aktuelles Kommanddo von Bus
Dim Wert As Single                                                    'Parameter zu Kommando
Dim Teiler As Integer                                                 'Teiler fr den Schritttimer
Dim Notaus_status As Byte                                             'Zeigt an ob der Notaus gedrckt ist
Dim Timer_teiler As Integer                                           'Frequenzteiler
Dim Modus As Byte                                                     'Gibt den Fahrmodul zurck
Dim Stufenschritte As Byte                                            'Die Anzahl Schritte die pro Stufe der Beschleunigungskurve gemacht werden
Dim Stufe As Single                                                   'Stufenwert der Beschleunigungsrampe



'Startwerte
Timer0 = Timervorgabe0
Timer1 = Timervorgabe1
Enable Interrupts                                                     'Interrupts (Timer und externe) zulassen

Pwm2 = 255                                                            'Strom auf 100%

Set Motorenable                                                       'Leitungsteil freigeben, Antwort kommt nach 500ms auf den Ready Leitungen
Set Motorgate                                                         'Pulse an Steuerung freigeben

Modulname = "Simo: "                                                  'Modulname setzten


'Auf Ready von Steuerung warten
Call Wait_ready


'Hauptprogramm
Do
  Input Buffer Noecho
  Call Interpreter
Loop


'Not-Aus Handler
Isr_not_aus:
  'Fahrtimer stoppen
  Disable Timer0

  'Zu fahrende Schritte lschen
  Schritte = 0

  'Motor stromlos
  Reset Motorenable

  Notaus_status = 1
  Print "Not-Aus gedrckt"

  'Statusrelais aktualisieren
  If Relaisfunktion = 3 Then Set Statusrelais                         'Not-Aus gedrckt, wird signalisiert
  If Relaisfunktion = 4 Then Reset Statusrelais                       'Not-Aus gedrckt, also nix bereit
Return


'ext. INT1 Handler
Isr_int1:

Return


'Timer0 (Fahrtimer) Handler
Isr_timer0:
  'Timer wieder laden
  Timer0 = Timervorgabe0

  'Timerinterupts teilen
  Incr Teiler

  'Wenn teiler erreicht wurde
  If Teiler >= Timer_teiler Then
    'Motorpuls absetzten

    'Pulspin setzen
    Set Motorpuls

    'Wenn nicht dauerlauf, dann Schritt abziehen
    If Modus = 0 Then Decr Re_schritte
    'Teiler lschen damit von neuem gezhlt werden muss
    Teiler = 0

    'Wenn gewnscht bei drehendem Motor Statusrelais setzen
    If Relaisfunktion = 1 Then Set Statusrelais
    'Der Motor dreht, also Statusrelais lschen
    If Relaisfunktion = 2 Then Reset Statusrelais
  End If

  'Sind noch Schritte zu fahren? Wenn nicht dann...
  If Re_schritte = 0 Then
    'Timer stoppen
    Disable Timer0
    'Statusrelais bei Motordreht, dann Relais jetzt lschen
    If Relaisfunktion = 1 Then Reset Statusrelais
    'Wenn der Motor steht und gewnscht Statusrelais setzten
    If Relaisfunktion = 2 Then Set Statusrelais
  End If

  'Pulspin lschen
  Reset Motorpuls
Return


'Timer1 (Sekundentimer) Handler
Isr_timer1:
  'Timer wieder laden
  Timer1 = Timervorgabe1

  'RunLed bedienen
  Toggle Runled

  If Notaus = 1 And Notaus_status = 1 Then
    Print "Not-Aus nicht mehr gedrckt"
    Notaus_status = 0
    'Motor wieder strom geben
    Set Motorenable
    'Auf Motorready warten
    Call Wait_ready
    'Statusrelais auf Not-Aus und Not-Aus nicht gedrckt, dann lschen
    If Relaisfunktion = 3 Then Reset Statusrelais
  End If

Return


'Befehlinterpreter fr Buskommandos
Sub Interpreter()
  Laenge = Len(modulname)
  Modul = Left(buffer , Laenge)
  If Modul = Modulname Then
    'Seriellen String zerlegen

    'Kommando
    Laenge = Len(buffer) - Laenge
    Buffer = Right(buffer , Laenge)
    Laenge = Instr(buffer , " ")
    Laenge = Laenge - 1
    Kommando = Left(buffer , Laenge)
    Kommando = Lcase(kommando)

    'Wert
    Laenge = Len(buffer) - Laenge
    Laenge = Laenge - 1
    Buffer = Right(buffer , Laenge)
    Wert = Val(buffer)

'    'Debugausgabe
'    Print "M " ; Modul
'    Print "K " ; Kommando
'    Print "W " ; Wert

    Select Case Kommando
    Case "e"                                                          'Execute/Ausfhren
      'Zu fahrende Schritte kopieren
      Re_schritte = Schritte
      'Fahrmodus festlegen
      Modus = Wert
      'Timer starten
      Enable Timer0
      Print "OK"
    Case "s"                                                          'Steps = Schritte
      Schritte = Wert
      Print "OK"
    Case "c"                                                          'Current = Strom in %
      'PWM Wert errechnen (2.55 = 255/100 -> PWMmax / 100%)
      Pwm2 = Wert * 2.55
      Print "OK"
    Case "f"                                                          'Frequency = Frequenz in Umdrehungen/min
      Drehzahl = Wert
      'Teiler errechnen
      Timer_teiler = 6000 / Drehzahl
      Print "OK"
    Case "d"                                                          'Motorrichtung
      If Wert = 0 Then
        Reset Motordir
        Richtung = 0
      Else
        Set Motordir
        Richtung = 1
      End If
      Print "OK"
    Case "x"                                                          'Exit/Abbrechen, die Schritte werden gelscht
      Disable Timer0
      Print "OK"
    Case "r"                                                          'Statusrelaisfunktion festlegen
      Relaisfunktion = Wert
      Print "OK"
    Case "v"
      Print "Version: 1.3"                                            'Version der Software
      Print "OK"
    End Select
  End If
End Sub


'Auf bereit von Simodrive warten
Sub Wait_ready()
  Do
  Loop Until Motorready = 0                                           'Motorready invertiert
  'Statusrelais nur Bereit, dann setzten
  If Relaisfunktion = 4 Then Set Statusrelais                         'Wenn gewscht, bereit signalisieren

  'Ready mit Blinken quitieren
  Disable Timer1
  Reset Runled
  Waitms 200
  Set Runled
  Waitms 200
  Reset Runled
  Waitms 200
  Set Runled
  Waitms 200
  Reset Runled
  Enable Timer1
End Sub


